function [fit_fun, cost_tot, phi, rot, G, cos_phi] =cost(Vo, n_fiber, Data)
% Definition of objective function, constraints and fitness function
%
% Inputs:
%     Vo         Vector of optimization variables
%     n_fiber    Number of fibers to be considered in mpfa_rcsde.m
%     Data       Vector that contains the following information:
%                [steelcost concretecost shutteringcost ksh nd mdx mdy ...
%                 fc fy cost_max_0 dpx dpy arm_min duct design_code ...
%                 spiral c1 c2 Max];
%
% Outputs:
%     fit_fun    Fitness function
%     cost_tot   Objective function (total cost)
%     phi        Strength reduction factor (ACI318)
%     rot        Section gyration
%     G          Vector of constraints
%     cos_phi    Cosine of angle (2nd strength constraint)
%
% Functions:
%     mpfa_rcsde.m
% _________________________________________________________________________

    steelcost      = Data(1,  1);
    concretecost   = Data(1,  2);
    shutteringcost = Data(1,  3);
    ksh            = Data(1,  4);
    nd             = Data(1,  5);
    mxd            = Data(1,  6);
    myd            = Data(1,  7);
    fc             = Data(1,  8);
    fy             = Data(1,  9);
    cost_max_0     = Data(1, 10);
    dpx            = Data(1, 11);
    dpy            = Data(1, 12);
    arm_min        = Data(1, 13);
    duct           = Data(1, 14);
    design_code    = Data(1, 15);
    spiral         = Data(1, 16);
    c1             = Data(1, 17);
    c2             = Data(1, 18);

    datalength=length(Data);    
    if datalength == 24
        Max     = Data(1, 19:24);
        f_opt   = Vo(1,1);
        teta    = Vo(1,2).*Max(1,1);
        b       = Vo(1,3).*Max(1,2);
        h       = Vo(1,4).*Max(1,3);
        phi_py  = Vo(1,5).*Max(1,4);
        n_py    = Vo(1,6).*Max(1,5);
        phi_y   = Vo(1,5).*Max(1,4);
        n_y     = Vo(1,6).*Max(1,5);
        phi_px  = Vo(1,5).*Max(1,4);
        n_px    = Vo(1,7).*Max(1,6);
        phi_x   = Vo(1,5).*Max(1,4);
        n_x     = Vo(1,7).*Max(1,6);   
    elseif datalength == 25
        Max     = Data(1, 19:25);
        f_opt   = Vo(1,1);
        teta    = Vo(1,2).*Max(1,1);
        b       = Vo(1,3).*Max(1,2);
        h       = Vo(1,4).*Max(1,3);
        phi_py  = Vo(1,5).*Max(1,4);
        n_py    = Vo(1,6).*Max(1,5);
        phi_y   = Vo(1,5).*Max(1,4);
        n_y     = Vo(1,6).*Max(1,5);
        phi_px  = Vo(1,7).*Max(1,6);
        n_px    = Vo(1,8).*Max(1,7);
        phi_x   = Vo(1,7).*Max(1,6);
        n_x     = Vo(1,8).*Max(1,7);   
    elseif datalength == 29        
        Max     = Data(1, 19:29);
        f_opt   = Vo(1,1);
        teta    = Vo(1,2)  .* Max(1,1);
        b       = Vo(1,3)  .* Max(1,2);
        h       = Vo(1,4)  .* Max(1,3);
        phi_py  = Vo(1,5)  .* Max(1,4);
        n_py    = Vo(1,6)  .* Max(1,5);
        phi_y   = Vo(1,7)  .* Max(1,6);
        n_y     = Vo(1,8)  .* Max(1,7);
        phi_px  = Vo(1,9)  .* Max(1,8);
        n_px    = Vo(1,10) .* Max(1,9);
        phi_x   = Vo(1,11) .* Max(1,10);
        n_x     = Vo(1,12) .* Max(1,11);   
    end
    
    f_d = tan((pi / 2) * f_opt) * 1000;  % depth of neutral axis in mm
    Steelarea = (pi / 4) * (n_py * phi_py ^ 2 + n_y * phi_y ^ 2 + ...
                            n_px * phi_px ^ 2 + n_x * phi_x ^ 2);
    cost_tot = steelcost * Steelarea + concretecost * ...
               (b * h - Steelarea)+ shutteringcost * (ksh * b + 2 * h);
    fo = cost_tot / cost_max_0;  % normalized objective function cost
%   Nominal loads or strength of current design
    [Nn,Mxn,Myn,rot,f_duct,d,~,As,Asp,yrLN,XR] = ...
        mpfa_rcsde(n_fiber, f_d, teta, b, h, phi_y, n_y, phi_py, n_py,...
                                        phi_x, n_x, phi_px, n_px,...
                                        dpx, dpy, fc, fy, design_code);

    dpx_geom = dpx;
    dpy_geom = dpy;
    dpx      = dpx + 0.5 * max([phi_py phi_y phi_px phi_x]);
    dpy      = dpy + 0.5 * max([phi_px phi_x]);

%   Reinforcement constraints
    f_ck = fc;
    f_yk = fy;
    if design_code == 1
       if f_ck <= 50
           f_ctm = 0.3 * (f_ck) ^ (2/3);
       else
           f_cm = f_ck + 8;
           f_ctm = 2.12 * log(1 + (f_cm / 10));
       end
       As_min = max([0.26*(f_ctm/f_yk)*b*(h-dpy) 0.0013*b*(h-dpy) ...
                     0.26*(f_ctm/f_yk)*h*(b-dpx) 0.0013*h*(b-dpx)]);
    elseif design_code == 2  
        As_min = max([0.25*sqrt(f_ck)*b*(h-dpy)/f_yk 1.4*b*(h-dpy)/f_yk ...
                      0.25*sqrt(f_ck)*h*(b-dpx)/f_yk 1.4*h*(b-dpx)/f_yk]);
    end
    g_As = (As / As_min)-1;
    if or(g_As > 0,~arm_min), g_As = 0; end

    if design_code == 1
        if nd > 0
            Asp_min = 0.05 * (nd * 1e3) / (f_yk / 1.15);
        else
            Asp_min = 1e-10;
        end
    elseif design_code == 2
        Asp_min = 1e-10;
    end
    g_Asp = (Asp / Asp_min)-1;
    if or(g_Asp > 0,~arm_min), g_Asp = 0; end

    if design_code == 1
        if nd > 0
            AsT_min = max([0.1 * (nd*1e3) / (f_yk/1.15) 0.002 * b * h]);
        elseif nd < 0
           AsT_min = b * h * f_ctm / (f_yk/1.15);
        else
            AsT_min = 1e-10;
        end
    elseif design_code == 2
        if nd > 0
            AsT_min = 0.01 * b * h;
        else
            AsT_min = 1e-10;
        end
    end    
    g_AsT = ((As+Asp) / AsT_min)-1;
    if or(g_AsT > 0,~arm_min), g_AsT = 0; end    

%   Strength constraints
    if design_code == 1     
        phi=1.0;
    elseif design_code == 2  
        if f_d/d > 0.6
            if spiral
                phi = 0.75;
            else
                phi = 0.65;
            end
        elseif f_d/d > 0.375
            if spiral
                phi = 0.75 + 0.15 * (d / f_d-(5/3));
            else
                phi = 0.65 + 0.25 * (d / f_d-(5/3));
            end
        else
            phi = 0.9;
        end
    end

    P = [nd mxd myd]';
    mod_P = sqrt(nd ^ 2 + mxd ^ 2 + myd ^ 2);
    unit_P = P ./ mod_P;
    Nn = Nn/1e3;       % kN
    Mxn = Mxn/1e6;     % kN x m
    Myn = Myn/1e6;     % kN x m
    NRd = Nn * phi;    % NRd  (axial strength)
    MxRd = Mxn * phi;  % MxRd (flexural strength, x axis)
    MyRd = Myn * phi;  % MyRd (flexural strength, y axis)
    PRd = [NRd MxRd MyRd]';
    mod_PRd=sqrt(NRd ^ 2 + MxRd ^ 2 + MyRd ^ 2);
    unit_PRd = PRd ./ mod_PRd;

    g_s1 = 1 - mod_P / mod_PRd;
    if g_s1 > 0, g_s1 = 0; end

    cos_phi = unit_P(1) .* unit_PRd(1) + unit_P(2) .* unit_PRd(2)+...
              unit_P(3) .* unit_PRd(3);
    n = exp(cos_phi/c1) - cos_phi/c2;
    g_s2 = -((1-cos_phi) ./ 2) .^ (1/n);
    if ~isreal(g_s2)
        g_s2 = real(g_s2);
    end
    if g_s2 > 0, g_s2 = 0; end

%   Ductility constraints
    if design_code == 1
        if f_ck <= 50
            f_duct = 0.25 * d;
        else
            f_duct = 0.15 * d;
        end
    end
    g_duct = 1 - f_d / f_duct;
    if or(g_duct > 0,~duct), g_duct = 0; end    

%   Steel reinforcement spacing constraints    
    s_x  = (b - 2 * dpx_geom - n_x * phi_x) / (n_x - 1);
    s_px = (b - 2 * dpx_geom - n_px * phi_px) / (n_px - 1);
    s_y  = (h - 2 * dpy_geom - n_y * phi_y) / (n_y - 1);
    s_py = (h - 2 * dpy_geom - n_py * phi_py) / (n_py - 1);

    if design_code == 1               
        smin_x  = max(phi_x, 20);
        smin_px = max(phi_px,20);
        smin_y  = max(phi_y, 20);
        smin_py = max(phi_py,20);
        smax_x = 1000000;
        smax_y = 1000000;
    elseif design_code == 2 
        if ksh == 1;
            smin_x  = max(phi_x, 25);    
            smin_px = max(phi_px,25);
            smin_y  = max(phi_y, 25);
            smin_py = max(phi_py,25);
        elseif ksh == 2
            smin_x  = max(1.5 * phi_x, 40);
            smin_px = max(1.5 * phi_px,40);
            smin_y  = max(1.5 * phi_y, 40);
            smin_py = max(1.5 * phi_py,40);
        end
        smax_x = min(380 * 280 / (2 * fy / 3) - 2.5 * dpx_geom, ...
                     300 * 280 / (2 * fy / 3));
        smax_y = min(380 * 280 / (2 * fy / 3) - 2.5 * dpy_geom, ...
                     300 * 280 / (2 * fy / 3));
    end
  
    g_esp_max_x  = 1 - s_x / smax_x;
    g_esp_max_y  = 1 - s_y / smax_y;
    g_esp_max_px = 1 - s_px / smax_x;
    g_esp_max_py = 1 - s_py / smax_y;

    yr1 = XR(2,1);
    yr2 = XR(2,2);
    yr3 = XR(2,3);
    yr4 = XR(2,4);

    if (yr1 > yrLN) && (yr4 > yrLN)
        g_esp_max_px = 0;
    end

    if (yr1 > yrLN) && (yr2 > yrLN)
        g_esp_max_py = 0;
    end

    if (yr1 > yrLN) && (yr2 > yrLN) && (yr4 > yrLN)
        g_esp_max_px = 0;
        g_esp_max_py = 0;
    end

    if (yr1 > yrLN) && (yr2 > yrLN) && (yr3 > yrLN) && (yr4 > yrLN)
        g_esp_max_x  = 0;
        g_esp_max_y  = 0;
        g_esp_max_px = 0;
        g_esp_max_py = 0;
    end

    if n_x  == 0,  g_esp_max_x   = 0; end    
    if n_px == 0,  g_esp_max_px  = 0; end    
    if n_y  == 0,  g_esp_max_y   = 0; end    
    if n_py == 0,  g_esp_max_py  = 0; end   

    if g_esp_max_x > 0,  g_esp_max_x  = 0; end    
    if g_esp_max_px > 0, g_esp_max_px = 0; end    
    if g_esp_max_y > 0,  g_esp_max_y  = 0; end    
    if g_esp_max_py > 0, g_esp_max_py = 0; end    

    g_esp_min_x  = s_x / smin_x - 1;
    g_esp_min_px = s_px / smin_px - 1;
    g_esp_min_y  = s_y / smin_y - 1;
    g_esp_min_py = s_py / smin_py - 1;

    if n_x  == 0,  g_esp_min_x   = 0; end    
    if n_px == 0,  g_esp_min_px  = 0; end    
    if n_y  == 0,  g_esp_min_y   = 0; end    
    if n_py == 0,  g_esp_min_py  = 0; end    

    if g_esp_min_x > 0,  g_esp_min_x  = 0; end    
    if g_esp_min_px > 0, g_esp_min_px = 0; end    
    if g_esp_min_y > 0,  g_esp_min_y  = 0; end    
    if g_esp_min_py > 0, g_esp_min_py = 0; end    

    g=g_As + g_Asp + g_AsT + g_duct + g_s2 + g_s1 + ...
      g_esp_max_x + g_esp_max_px + g_esp_max_y + g_esp_max_py + ...
      g_esp_min_x + g_esp_min_px + g_esp_min_y + g_esp_min_py;
    g = -g;    
    
    G=[g_As g_Asp g_AsT g_duct g_s2 g_s1 ...
       g_esp_max_x g_esp_max_px g_esp_max_y g_esp_max_py ...
       g_esp_min_x g_esp_min_px g_esp_min_y g_esp_min_py];

%   Fitness function
    fit_fun = fo + g * (fo + 1 / fo);
end
