function [VO,FIT_FUN,n_np] = mpfa_move(fhandle,n,VO,FIT_FUN,alpha,...
                                            beta0,betamin,gamma,m,Lb,Ub,...
                                            Data,n_fiber,exp_alpha,fp)
% Move fireflies towards those that are the brightest
%
% Notes:
%      1) The attractiveness 'betaij' between two fireflies (i and j) is:
%             betaij = (beta0 - betamin) / (1 + gamma * rij ^ m) + betamin;
%                 where rij = distance between two fireflies
%
%      2) The new position 'VO_i(t+1)' of firefly i is obtained from
%         the previous position 'VO_i(t)' of firefly i and 
%         the previous position 'VO_j(t)' of firefly j 
%         as follows:
%             VO_i(t+1) = VO_i(t) + (VO_i(t) - VO_j(t)) * betaij +
%                         alpha * random_step
%
%      3) Possible values of 'betaij':
%         i)   with a large distance of rij  ---> betaij = betamin
%         ii)  with a short distance of rij  ---> betaij = 1
%         iii) the higher the gamma value, the lower the betaij value
%         iv)  if gamma = 0 (full transparency)  --->   betaij = 1
%                  In this case, the displacement is maximum and 
%                  firefly i jumps towards firefly j (the brightest one)
%
% Functions:
%     nb_dsc.m
%     hb_dsc.m
%     phi_dsc.m
%     bounds.m
%     fhandle.m
% _________________________________________________________________________

    d      = length(Lb);
    Scale  = abs(Ub-Lb);
    n_np   = 0;
    for i = n : -1 : 2,   % the firefly i moves ...
        coef=0.7;
        for j = i-1 : -1 : 1,  % towards firefly j
            if FIT_FUN(i,1) > FIT_FUN(j,1),  % move if i is worse than j 
                rij = sqrt(sum((VO(i,:) - VO(j,:)) .^ 2));
                betaij = (beta0-betamin) * (1 / (1 + gamma * rij ^ m)) +...
                         betamin;
                coef=sin(pi*coef);
                random_step = (rand(1,d) - 0.5) .* Scale;
%               continuous values in VO_1
                VO_1(i,:) = VO(i,:) + (VO(j,:) - VO(i,:)) .* betaij +...
                            coef*alpha .* random_step;
%               but number of bars are discrete in VO_1    
                VO_1(i,:) = nb_dsc(VO_1(i,:), Data); 
%               number of bars are also discrete in VO_2 (so that always) 
%               but the rest of values are continuous in VO_2
                VO_2(i,:) = VO_1(i,:);
%               h and b are discrete in VO_1    
                VO_1(i,:) = hb_dsc(VO_1(i,:), Data);
%               bar diameters are discrete in VO_1    
                VO_1(i,:) = phi_dsc(VO_1(i,:), Data);
%               weight discrete (VO_1) and continuous (VO_2) values
                VO_1(i,:) = (1-alpha) .* VO_1(i,:) + alpha .* VO_2(i,:);
%               make sure coordinates are within the bounds Lb/Ub                
                VO(i,:) = bounds(1, VO_1(i,:), Lb, Ub);
%               call the objective function                
                FIT_FUN(i,1) = fhandle(VO(i,:), n_fiber, Data);
                n_np = n_np + 1;
%               check whether the firefly i becomes the best during motion
                if FIT_FUN(i,1) < FIT_FUN(1,1) 
                    Vo_best = VO(i,:);
                    Fit_fun_best = FIT_FUN(i,1); 
%                   rank fireflies
                    for k = i-1 : -1 : 1    
                        VO(k+1,:) = VO(k,:);
                        FIT_FUN(k+1,1) = FIT_FUN(k,1);
                    end
                    VO(1,:) = Vo_best;
                    FIT_FUN(1,1) = Fit_fun_best;
%                   Local Small Random Displacement                     
                    VO(1,1) = VO(1,1) + fp * (alpha ^ exp_alpha) * ...
                             (rand(1,1) - 0.5) * abs(Ub(1) - Lb(1));
                    VO(1,2) = VO(1,2) + fp * (alpha ^ exp_alpha) *...
                             (rand(1,1) - 0.5) * abs(Ub(2) - Lb(2));
                end
            end
        end
    end
    [VO(:,:)] = bounds(n, VO(:,:), Lb, Ub);
end    
